package edu.nyu.scps.tree;

import java.util.Stack;

import android.app.ListActivity;
import android.content.Context;
import android.content.Intent;
import android.os.Bundle;
import android.view.KeyEvent;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.AdapterView;
import android.widget.AdapterView.OnItemClickListener;
import android.widget.BaseAdapter;
import android.widget.LinearLayout;
import android.widget.LinearLayout.LayoutParams;
import android.widget.TextView;

public class TreeActivity extends ListActivity {
	Tree tree = new Tree("America", new Tree[] {
		new Tree("Connecticut", new Tree[] {
			new Tree("Fairfield", new Tree[] {
				new Tree("Cos Cob", new Tree[] {		
				})		
			})	
		}),

		new Tree("New York", new Tree[] {
			new Tree("Westchester", new Tree[] {
				new Tree("Yonkers", new Tree[] {		
				}),
				new Tree("Hastings", new Tree[] {		
				}),
				new Tree("Dobbs Ferry", new Tree[] {		
				}),
				new Tree("Irvington", new Tree[] {		
				}),
				new Tree("Tarrytown", new Tree[] {		
				}),
				new Tree("Ossining", new Tree[] {		
				}),
				new Tree("Croton", new Tree[] {		
				}),
				new Tree("Peekskill", new Tree[] {		
				})
			}),
			new Tree("Putnam", new Tree[] {
				new Tree("Manitou", new Tree[] {		
				}),
				new Tree("Garrison", new Tree[] {		
				}),
				new Tree("Cold Spring", new Tree[] {		
				})
			}),
			new Tree("Dutchess", new Tree[] {
				new Tree("Beacon", new Tree[] {		
				}),
				new Tree("New Hamburg", new Tree[] {		
				}),
				new Tree("Poughkeepsie", new Tree[] {		
				})	
			}),
			new Tree("Columbia", new Tree[] {
			}),
			new Tree("Rensselaer", new Tree[] {
			})
		}),

		new Tree("New Jersey", new Tree[] {
			new Tree("Bergen", new Tree[] {
				new Tree("Fort Lee", new Tree[] {			
				}),
				new Tree("Alpine", new Tree[] {			
				})
			}),
		}),

		new Tree("Pennsylvania", new Tree[] {
		}),
	});

	Stack<Tree> stack = new Stack<Tree>();

	/** Called when the activity is first created. */
	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		//setContentView(R.layout.main);

		stack.push(tree);
		update();

		getListView().setOnItemClickListener(new OnItemClickListener() {
			@Override
			public void onItemClick(AdapterView<?> parent, View view, int position, long id) {
				//The tree that was clicked on.
				Tree tree = stack.peek().subtree[position];

				if (tree.subtree.length == 0) {
					//This tree is a leaf.
					Intent intent = new Intent(TreeActivity.this, WikiActivity.class);
					intent.putExtra("name", tree.name);
					startActivity(intent);
				} else {
					stack.push(tree);
					update();
				}
			}
		});
	}

	//Called after a push or pop.
	//The list contained by the adapter is always non-empty.

	private void update() {
		Tree top = stack.peek();
		getActionBar().setTitle(top.name);

		MyAdapter adapter = new MyAdapter(top);
		setListAdapter(adapter);
		adapter.notifyDataSetChanged();	//Redisplay the ListView.
	}

	@Override
	public boolean onKeyDown(int keyCode, KeyEvent keyEvent) {
		if (keyCode == KeyEvent.KEYCODE_BACK && keyEvent.getAction() == KeyEvent.ACTION_DOWN) {
			//The user pressed the Android back button.
			if (stack.size() > 1) {
				stack.pop();
				update();
			}
			return true;
		}
		return false;
	}

	private class Tree {
		public String name;
		public Tree[] subtree;

		public Tree (String name, Tree[] subtree) {
			this.name = name;
			this.subtree = subtree;
		}
	}

	//A MyAdapter does not display the entire tree of America.
	//It displays only one designated subtree (e.g., Westchester).

	private class MyAdapter extends BaseAdapter {
		Tree tree;

		public MyAdapter(Tree tree) {
			this.tree = tree;
		}

		@Override
		public int getCount() {
			return tree.subtree.length;
		}

		@Override
		public Object getItem(int position) {
			return tree.subtree[position];
		}

		@Override
		public long getItemId(int position) {
			return position;
		}

		@Override
		public View getView(int id, View convertView, ViewGroup viewGroup) {
			LinearLayout linearLayout;

			if (convertView == null) {
				LayoutInflater inflater = (LayoutInflater)TreeActivity.this.getSystemService(Context.LAYOUT_INFLATER_SERVICE);
				TextView textView1 = (TextView)inflater.inflate(android.R.layout.simple_list_item_1, viewGroup, false);
				textView1.setLayoutParams(new LinearLayout.LayoutParams(0, LayoutParams.WRAP_CONTENT, 1f));

				TextView textView2 = (TextView)inflater.inflate(android.R.layout.simple_list_item_1, viewGroup, false);
				textView2.setLayoutParams(new LinearLayout.LayoutParams(LayoutParams.WRAP_CONTENT, LayoutParams.WRAP_CONTENT, 0f));
				//">" looks like iOS UITableViewCellAccessoryDisclosureIndicator chevron.
				textView2.setText(">");

				linearLayout = new LinearLayout(TreeActivity.this);
				linearLayout.setOrientation(LinearLayout.HORIZONTAL);
				linearLayout.addView(textView1);
				linearLayout.addView(textView2);
			} else {
				linearLayout = (LinearLayout)convertView;
			}

			((TextView)linearLayout.getChildAt(0)).setText(tree.subtree[id].name);
			int visibility = tree.subtree[id].subtree.length > 0 ? View.VISIBLE : View.GONE;
			((TextView)linearLayout.getChildAt(1)).setVisibility(visibility);
			return linearLayout;
		}
	}
}